@@ -43,4 +43,5 @@ if __name__ == '__main__':
     docwriter.write_index(outdir, 'gen',
                           relative_to = pjoin('source','api')
                           )
-    print '%d files written' % len(docwriter.written_modules)
+    
+    print('%d files written' % len(docwriter.written_modules))
