@@ -1,6 +1,6 @@
 # -*- coding: utf8 -*-
 #-----------------------------------------------------------------------------
-#  Copyright (c) 2010-2012 Brian Granger, Min Ragan-Kelley
+#  Copyright (c) 2010 Brian Granger, Min Ragan-Kelley
 #
 #  This file is part of pyzmq
 #
@@ -15,10 +15,11 @@
 import sys
 import time
 import errno
+import warnings
 
 import zmq
 from zmq.tests import (
-    BaseZMQTestCase, SkipTest, have_gevent, GreenTest, skip_pypy
+    BaseZMQTestCase, SkipTest, have_gevent, GreenTest, skip_pypy, skip_if
 )
 from zmq.utils.strtypes import bytes, unicode
 
@@ -68,6 +69,13 @@ class TestSocket(BaseZMQTestCase):
         except zmq.ZMQError as e:
             self.assertEqual(e.errno, zmq.EPROTONOSUPPORT)
 
+    def test_identity(self):
+        s = self.context.socket(zmq.PULL)
+        self.sockets.append(s)
+        ident = b'identity\0\0'
+        s.identity = ident
+        self.assertEqual(s.get(zmq.IDENTITY), ident)
+
     def test_unicode_sockopts(self):
         """test setting/getting sockopts with unicode strings"""
         topic = "tést"
@@ -84,8 +92,10 @@ class TestSocket(BaseZMQTestCase):
         self.assertRaises(TypeError, s.getsockopt_unicode, zmq.AFFINITY)
         self.assertRaisesErrno(zmq.EINVAL, s.getsockopt_unicode, zmq.SUBSCRIBE)
         
-        st = s.getsockopt(zmq.IDENTITY)
-        self.assertEqual(st.decode('utf16'), s.getsockopt_unicode(zmq.IDENTITY, 'utf16'))
+        identb = s.getsockopt(zmq.IDENTITY)
+        identu = identb.decode('utf16')
+        identu2 = s.getsockopt_unicode(zmq.IDENTITY, 'utf16')
+        self.assertEqual(identu, identu2)
         time.sleep(0.1) # wait for connection/subscription
         p.send_unicode(topic,zmq.SNDMORE)
         p.send_unicode(topic*2, encoding='latin-1')
@@ -93,11 +103,9 @@ class TestSocket(BaseZMQTestCase):
         self.assertEqual(topic*2, s.recv_unicode(encoding='latin-1'))
     
     def test_int_sockopts(self):
-        "test non-uint64 sockopts"
+        "test integer sockopts"
         v = zmq.zmq_version_info()
-        if not v >= (2,1):
-            raise SkipTest("only on libzmq >= 2.1")
-        elif v < (3,0):
+        if v < (3,0):
             default_hwm = 0
         else:
             default_hwm = 1000
@@ -127,7 +135,10 @@ class TestSocket(BaseZMQTestCase):
                 backref[value] = name
         for opt in zmq.constants.int_sockopts.union(zmq.constants.int64_sockopts):
             sopt = backref[opt]
-            if sopt.startswith(('ROUTER', 'XPUB', 'TCP')):
+            if sopt.startswith((
+                'ROUTER', 'XPUB', 'TCP', 'FAIL',
+                'REQ_', 'CURVE_', 'PROBE_ROUTER',
+                )):
                 # fail_unroutable is write-only
                 continue
             try:
@@ -221,9 +232,17 @@ class TestSocket(BaseZMQTestCase):
 
         b.bind(iface)
         msg = b.recv_multipart()
+        for i in range(10):
+            if p1.done:
+                break
+            time.sleep(0.1)
         self.assertEqual(p1.done, True)
         self.assertEqual(msg, [b'something'])
         msg = b.recv_multipart()
+        for i in range(10):
+            if p2.done:
+                break
+            time.sleep(0.1)
         self.assertEqual(p2.done, True)
         self.assertEqual(msg, [b'something', b'else'])
         m = zmq.Frame(b"again", track=True)
@@ -243,7 +262,10 @@ class TestSocket(BaseZMQTestCase):
         self.assertEqual(p2.done, False)
         pm = m.tracker
         del m
-        time.sleep(0.1)
+        for i in range(10):
+            if p1.done:
+                break
+            time.sleep(0.1)
         self.assertEqual(p1.done, True)
         self.assertEqual(p2.done, True)
         m = zmq.Frame(b'something', track=False)
@@ -390,5 +412,24 @@ if have_gevent:
             timeout.start()
             self.assertRaises(gevent.Timeout, b.recv)
             g.kill()
+        
+        @skip_if(not hasattr(zmq, 'RCVTIMEO'))
+        def test_warn_set_timeo(self):
+            s = self.context.socket(zmq.REQ)
+            with warnings.catch_warnings(record=True) as w:
+                s.rcvtimeo = 5
+            s.close()
+            self.assertEqual(len(w), 1)
+            self.assertEqual(w[0].category, UserWarning)
+            
+
+        @skip_if(not hasattr(zmq, 'SNDTIMEO'))
+        def test_warn_get_timeo(self):
+            s = self.context.socket(zmq.REQ)
+            with warnings.catch_warnings(record=True) as w:
+                s.sndtimeo
+            s.close()
+            self.assertEqual(len(w), 1)
+            self.assertEqual(w[0].category, UserWarning)
 
 
